import os


CONFIG_FILE_ASKANNA = os.path.expanduser("~/.askanna.yml")
CONFIG_ASKANNA_REMOTE = {"askanna": {"remote": "https://beta-api.askanna.eu/v1/"}}

DEFAULT_PROJECT_TEMPLATE = "https://gitlab.askanna.io/open/project-templates/blanco-template.git"

PYPI_PROJECT_URL = "https://pypi.org/pypi/askanna/json"
