class Error(Exception):
    pass


class GetError(Error):
    pass


class HeadError(Error):
    pass


class PostError(Error):
    pass


class PatchError(Error):
    pass


class PutError(Error):
    pass


class RunError(Error):
    pass
